# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
#
# OGIGameUtils - Callback Functions
# Implementaion of miscellaneous game functions

from CvPythonExtensions import *
import CvUtil
import CvEventInterface
import CvGameUtils
import Popup as PyPopup
import PyHelpers
import IceBreaker
import StandardFunctions

# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame


class OGIGameUtils(CvGameUtils.CvGameUtils):
	"Miscellaneous game functions"
	def __init__(self): 
		pass
	
	
	def AI_chooseProduction(self,argsList):
		pCity = argsList[0]
		iOwner = pCity.getOwner( )
		AIpPlayer = gc.getPlayer(iOwner)
		iStateReligion = AIpPlayer.getStateReligion()
		iUnitTypeToProduce = -1
		bProduceUnit = False
		
		if not bProduceUnit:
			# Ice Breaker Mod
			if not gc.getGame().isOption(GameOptionTypes.GAMEOPTION_OGI_NO_ICE_BREAKERS):
				iCoastGuard = gc.getInfoTypeForString("TECH_COAST_GUARD")
				
				if StandardFunctions.hasTechCheck(iOwner, iCoastGuard):
					# Is this a coastal city?
					if pCity.isCoastal(gc.getMIN_WATER_SIZE_FOR_OCEAN()):
						#if pCity.hasIceBreakerPlot():
						if pCity.getPopulation() > 4:
							iIceBreaker = gc.getInfoTypeForString("UNIT_ICE_BREAKER")
							
							if pCity.canTrain(iIceBreaker, 0, 0):
								iIceBreakerUnitClassType = gc.getInfoTypeForString("UNITCLASS_ICE_BREAKER")
								#iIceBreakerUnitClassType = gc.getUnitInfo(iIceBreaker).getUnitClassType()
								iMaxUnits = gc.getUnitClassInfo(iIceBreakerUnitClassType).getMaxPlayerInstances()
								iTotalUnitClassCount = AIpPlayer.getUnitClassCount(iIceBreakerUnitClassType)
								
								# if player has less Ice Breakers than the maximum allowed, then build one.
								if iTotalUnitClassCount < iMaxUnits:
									#intIceBreakerChance = IceBreaker.getIceBreakerProductionChance(AIpPlayer)
									intIceBreakerChance = 33
									# Roll the dice!
									if StandardFunctions.doRandumChance(intIceBreakerChance):
										iUnitTypeToProduce = iIceBreaker
										bProduceUnit = True
										# Ice Breaker Mod
		
		if bProduceUnit:
			# Make the City produce the Unit
			pCity.pushOrder(OrderTypes.ORDER_TRAIN, iUnitTypeToProduce, -1, False, False, False, True)
			#CyInterface().addImmediateMessage("Unit is on Order", "")
			return True
		
		return False
	
	
	def AI_unitUpdate(self,argsList):
		pUnit = argsList[0]
		# Orion's Inquisition Mod
		iOwner = pUnit.getOwner()
		AIpPlayer = gc.getPlayer(iOwner)
		pUnitType = pUnit.getUnitType()
		
		if not AIpPlayer.isNone() and not AIpPlayer.isBarbarian() and not AIpPlayer.isHuman() and AIpPlayer.isAlive():
			# Ice Breaker Mod
			if not gc.getGame().isOption(GameOptionTypes.GAMEOPTION_OGI_NO_ICE_BREAKERS):
				if pUnitType == gc.getInfoTypeForString("UNIT_ICE_BREAKER"):
					pUnit.doAIIceBreakerPlacement()
					return True
					# End Ice Breaker Mod
		
		return False
# End Ice Breaker Mod
