# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
# Orion's Standard Functions
from CvPythonExtensions import *
import CvUtil
import CvTechChooser
import traceback
import os
import sys
import PyHelpers
import pickle
import CvGameUtils
import CvEventInterface
import CvWBPopups
import Popup as PyPopup

# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame


# ###################################
#	Begin Standard Functions		#
# ###################################
def CvHasBuildingClass(iPlayer, strBuildingClass):
	# Orion's Standard Functions
	# Returns True if player has the specified Building
	# Returns False if player does not have the specified Building
	# Use this function for single occurance buildings (Wonders)
	pPlayer = gc.getPlayer(iPlayer)
	iBCBuildingClass = gc.getInfoTypeForString(strBuildingClass)
	HasStrBuildingClass = False
	
	if (pPlayer.getBuildingClassCount(iBCBuildingClass)>0):
		HasStrBuildingClass = True
	
	return HasStrBuildingClass


def CvHasBuilding(iPlayer, strBuilding):
	# Orion's Standard Functions
	# Returns True if player has the specified Building
	# Returns False if player does not have the specified Building
	pPlayer = gc.getPlayer(iPlayer)
	iRefCityList = PyPlayer(iPlayer).getCityList()
	HasStrBuilding = False
	
	for pyCity in iRefCityList:
		pCity = pyCity.GetCy()
		
		if pCity.isHasBuilding(gc.getInfoTypeForString(strBuilding)):
			#CyInterface().addMessage(CyGame().getActivePlayer(),True,25,'Found Building','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),0,0,False,False)
			HasStrBuilding = True
			break
			
	return HasStrBuilding
	
	
def isBuildingObsolete(iPlayer, strBuilding):
	# Orion's Standard Functions
	# Returns True if the obsolete Tech for a specified building has been discovered
	# Returns False if obsolete Tech for a specified building has not been discovered
	pPlayer = gc.getPlayer(iPlayer)
	Building = gc.getInfoTypeForString(strBuilding)
	obsoleteTech = gc.getBuildingInfo(Building).getObsoleteTech()
	
	if ( gc.getTeam(pPlayer.getTeam()).isHasTech(obsoleteTech) == False or obsoleteTech == -1 ):
		return False
	
	return True
	

def getSpecialReligiousBuilding(iPlayer, iReligion, strSpecialBuilding):
	# Orion's Standard Functions
	# Works only for the following Buildings: Monastery, Temple, Cathedral, HolyOffice and Shrine
	# Returns the specified Special Building Name for the specified Religion
	pPlayer = gc.getPlayer(iPlayer)
	pCivilization = gc.getCivilizationInfo(pPlayer.getCivilizationType())
	
	if strSpecialBuilding == "Monastery":
		iSpecialBuilding = gc.getInfoTypeForString("SPECIALBUILDING_MONASTERY")
	
	if strSpecialBuilding == "Temple":
		iSpecialBuilding = gc.getInfoTypeForString("SPECIALBUILDING_TEMPLE")
	
	if strSpecialBuilding == "Cathedral":
		iSpecialBuilding = gc.getInfoTypeForString("SPECIALBUILDING_CATHEDRAL")
		
	if strSpecialBuilding == "HolyOffice":
		iSpecialBuilding = gc.getInfoTypeForString("SPECIALBUILDING_HOLY_OFFICE")
	
	for iBuildingClass in range(gc.getNumBuildingClassInfos()):
		kBuilding = gc.getBuildingInfo(pCivilization.getCivilizationBuildings(iBuildingClass))
		
		if kBuilding != None:
			if strSpecialBuilding == "Shrine":
				if gc.getBuildingClassInfo(iBuildingClass).getMaxGlobalInstances() > 0:
					if kBuilding.getGlobalReligionCommerce() == iReligion:
						if kBuilding.getHolyCity() == iReligion:
							# Get the Shrine Name
							iBuilding = pCivilization.getCivilizationBuildings(iBuildingClass)
							MyShrineName = gc.getBuildingInfo(iBuilding).getType()
							#CyInterface().addImmediateMessage(str(MyShrineName), "")
							return MyShrineName
							break
			
			elif strSpecialBuilding != "Shrine":
				# Look for Prerequisite Religion
				if kBuilding.getPrereqReligion() != -1:
					# Prerequisite Religion
					iBldPrereqRel = kBuilding.getPrereqReligion()
					
					if iBldPrereqRel == iReligion:
						iBuilding = pCivilization.getCivilizationBuildings(iBuildingClass)
						
						if gc.getBuildingInfo(iBuilding).getSpecialBuildingType() == iSpecialBuilding:
							MyReligiousBuildingName = gc.getBuildingInfo(iBuilding).getType()
							#CyInterface().addImmediateMessage(str(MyReligiousBuildingName), "")
							return MyReligiousBuildingName
							break


def plotHasFeature(pPlot, intFeature):
	# Orion's Standard Functions
	# Returns True if the plot has the specified feature.
	bFoundFeature = False
	
	for iFeature in range(gc.getNumFeatureInfos()):
		if pPlot.getFeatureType() == intFeature:
			bFoundFeature = True
			break
	
	return bFoundFeature
	
	
def isSinglePlotIsland(testPlot):
	# Orion's Standard Functions
	# Checks each adjacent plots around the specified plot
	# Tests the plot for water (Terrain coast and Terrain Ocean)
	bWaterPlot = True
	# Limit iDirection to 8
	iDirection = CyGame().getSorenRandNum(8,"")
	
	# Loop through adjacent plots
	for iDirection in range(DirectionTypes.NUM_DIRECTION_TYPES):
		pAdjacentPlot = plotDirection( iX, iY, DirectionTypes(iDirection))
		pAdjacentPlotiOwner = pAdjacentPlot.getOwner()
		
		# make sure plot is not None
		if not pAdjacentPlot.isNone():
			# Exclude the plot the Icebreaker is currently on.
			if pAdjacentPlot != testPlot:
				if not testPlot.isWater():
					bWaterPlot = False
	
	return bWaterPlot
	
	
def isHasFeature(SourcePlot, iFeature):
	# Orion's Standard Functions
	# Checks plots around the specified source plot
	# Distance is in increments of 1 
	# Max distance to check is 10 plots away
	# Returns True if plot has specified feature
	FoundTargetIcePlot = False
	
	for iPlotDistance in range(2, 10, 1):
		if FoundTargetIcePlot:
			break
		
		else:
			for dx in range(-1, iPlotDistance):
				for dy in range(-1, iPlotDistance):
					# ignore same tile. Adjacent-tile healing does not work on the same tile.
					if not (dx == 0 and dy == 0):
						pFeaturePlot = CyMap().plot(SourcePlot.getX()+dx, SourcePlot.getY()+dy)
						if pFeaturePlot.getFeatureType() == iFeature:
							FoundTargetIcePlot = True
							#CyInterface().addImmediateMessage("Found Feature!", "")
							break
							break
	
	return FoundTargetIcePlot
	
	
def findFeaturePlot(SourcePlot, iFeature):
	# Orion's Standard Functions
	# Checks plots around the specified source plot
	# Distance is in increments of 1 
	# Max distance to check is 10 plots away
	# Returns True if plot has specified feature
	FoundTargetIcePlot = False
	
	for iPlotDistance in range(2, 10, 1):
		if FoundTargetIcePlot:
			break
		
		else:
			for dx in range(-1, iPlotDistance):
				for dy in range(-1, iPlotDistance):
					# ignore same tile. Adjacent-tile healing does not work on the same tile.
					if not (dx == 0 and dy == 0):
						pFeaturePlot = CyMap().plot(SourcePlot.getX()+dx, SourcePlot.getY()+dy)
						if pFeaturePlot.getFeatureType() == iFeature:
							FoundTargetIcePlot = True
							#CyInterface().addImmediateMessage("Found Feature!", "")
							break
							break
	
	if FoundTargetIcePlot:
		return pFeaturePlot
	

def isTypeInList(xType, dList):
	# Orion's Standard Functions
	# Returns True if the xType is found in the dList
	# Returns False if the xType is not found in the dList
	bFoundType = False
	
	for szType in range(len(dList)):
		MyType = str(dList[szType])
		iType = gc.getInfoTypeForString(MyType)
		
		if xType == iType:
			#CyInterface().addImmediateMessage("Found It!", "")
			bFoundType = True
			break
	
	return bFoundType
	

def doRandumChance(Number):
	# Orion's Standard Functions
	# Produces a number between 0 and 100
	chance = CyGame().getSorenRandNum(100,"")
	
	if chance <= Number:
		return True
	else:
		return False


def isGroupPromotion(ipzPromotion):
	# Orion's Standard Functions
	# Returns True if a Promotion is a Group Promotion
	# Returns False if a Promotion is not a Group Promotion
	#Promotion = gc.getPromotionInfo(ipzPromotion).getDescription()
	Promotion = gc.getPromotionInfo(ipzPromotion).getType()
	if Promotion[-1] in map(str,range(10)):
		# Promotion is a Group Promotion
		#CyInterface().addImmediateMessage(str(Promotion), "")
		return True
	
	return False

	
def getPromotionRate(iMaxUnits, iTotalPromotions):
	# Orion's Standard Functions
	# Returns a calculated Promotion rate
	if iTotalPromotions == 0:
		MyRate = 0
		
	else:
		MyRate = abs(iTotalPromotions / iMaxUnits * .33333333333 * 100)
		
		if MyRate > 0:
			if MyRate > 100:
				MyRate = 100
				
	return str(MyRate)
	
	
def getTotalUnitPromotions(pUnit):
	# Orion's Standard Functions
	' Total count of Unit Promotions '
	iPromTotal = 0
	
	for iPromotion in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if pUnit.isHasPromotion(iPromotion):
			iPromTotal += 1
	
	return iPromTotal

	
def getPromotionLevel(iProm):
	# Orion's Standard Functions
	# Returns the Promotion Level for the specified Promotion
	if isGroupPromotion(iProm):
		Promotion = gc.getPromotionInfo(iProm).getType()
		ProLevel = Promotion[-1]
		return str(ProLevel)
	
	else:
		return str("1")
	
	
def getPromotionGroup(iProm):
	# Orion's Standard Functions
	# Strips off "PROMOTION_" and the Promotion level from the specified group Promotion
	# Returns the Promotion Group
	# Returns "NONE" for single promotions
	if isGroupPromotion(iProm):
		Promotion = gc.getPromotionInfo(iProm).getType()
		StripPromotion = Promotion.strip("PROMOTION_")
		ProGrp = StripPromotion.rstrip(str(StripPromotion[-1]))
		return str(ProGrp)
	
	else:
		return "NONE"
	
	
def getHighestPromotionLevel(pGroup):
	# Orion's Standard Functions
	# Returns the Highest Promotion Level for the specified promotion group
	iLevelCount = 0
	
	for iProm in range(gc.getNumPromotionInfos()):
		MyProGroup = getPromotionGroup(iProm)
		if MyProGroup == pGroup:
			iLevelCount += 1
	
	return iLevelCount
	
	
def runGroupListCheck(kUnit):
	# Orion's Standard Functions
	lGroups = list()
	
	for iProm in range(gc.getNumPromotionInfos()):
		strGroup = getPromotionGroup(iProm)
		# Setup a summary list of group promotions  
		if strGroup not in lGroups:
			lGroups.append(strGroup)
			# Does the Unit have this group promotion? 
			for iPr in range(gc.getNumPromotionInfos()):
				if kUnit.isHasPromotion(iPr):
					GroupName = getPromotionGroup(iPr)
					#CyInterface().addImmediateMessage(str(GroupName), "")
					if GroupName == strGroup:
						# Yup!  Run promotion group checks.
						doPromotionGroupRandumChecks(kUnit, strGroup)
						#CyInterface().addImmediateMessage("Here I am!", "")
						break


def doPromotionGroupRandumChecks(kUnit, kGroup):
	# Orion's Standard Functions
	iMax = getHighestPromotionLevel(kGroup)
	ContinueSearch = True
	#CyInterface().addImmediateMessage(str(iMax), "")
	
	for k in range(iMax, 0, -1):
		if ContinueSearch:
			for iPromotion in range(gc.getNumPromotionInfos()):
				#strGroup = getPromotionGroup(iPrm)
				if kUnit.isHasPromotion(iPromotion):
					iPromotionLevel = getPromotionLevel(iPromotion)
					if iPromotionLevel == k:
						iSinglePromotionRate = getPromotionRate(iTotalUnitsDivider, iTotalPromotions)
						
						if k == 6:
							# promotion level = Single Rate
							GroupChance = iSinglePromotionRate
							if not doRandumChance(iSinglePromotionRate):
								kUnit.setHasPromotion(iPromotion, false)
								break
						
						elif k == 5:
							# promotion level = Single Rate * 1.2
							GroupChance = iSinglePromotionRate * 1.2
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						
						elif k == 4:
							# promotion level = Single Rate * 1.4
							GroupChance = iSinglePromotionRate * 1.4
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						
						# For promotions at level 3
						elif k == 3:
							# promotion level = Single Rate * 1.6
							GroupChance = iSinglePromotionRate * 1.6
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						
						# For promotions at level 2
						elif k == 2:
							# promotion level = Single Rate * 1.8
							GroupChance = iSinglePromotionRate * 1.8
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, false)
								break
						
						# For promotions at level 1
						elif k == 1:
							iPrereq = gc.getPromotionInfo(iPromotion).getPrereqOrPromotion1()
							# promotion level = Single Rate * 2
							GroupChance = iSinglePromotionRate * 2
							
							if iPrereq != -1:
								# Make sure the prerequisite promotion was not deleted
								if not kUnit.isHasPromotion(iPrereq):
									kUnit.setHasPromotion(iPrereq, True)
							
							if not doRandumChance(GroupChance):
								kUnit.setHasPromotion(iPromotion, False)
								break
						
						else:
							ContinueSearch = False
							break
		
		else:
			break


def doTransferPromotions(FromUnit, ToUnit):
	# Orion's Standard Functions
	# Transfers promotions from selected unit to the combined unit
	iGreatGeneral = gc.getInfoTypeForString("UNIT_GREAT_GENERAL")
	
	# Loop through promotions
	for iProm in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if FromUnit.isHasPromotion(iProm):
			# Combined unit must not already have this promotion
			if not ToUnit.isHasPromotion(iProm):
				# Transfer promotion to the New Combined Unit
				ToUnit.setHasPromotion(iProm, True)
				# Get the Leader Type
				FromUnitLeaderType = FromUnit.getLeaderUnitType()
				# Does the unit have a leader?
				if FromUnitLeaderType != -1:
					# Get the Leader Name
					FromUnitName = FromUnit.getNameNoDesc()
					#CyInterface().addImmediateMessage(str(FromUnitName), "")
					# Get the Leader Type for Combined Unit
					MyCombinedUnitLeaderType = ToUnit.getLeaderUnitType()
					# Make sure the Army unit does not already have a leader
					if MyCombinedUnitLeaderType == -1:
						ToUnit.setLeaderUnitType(iGreatGeneral)
						ToUnit.setName(FromUnitName)


def isGreatLeader(CheckUnit):
	# Orion's Standard Functions
	# Returns True if the specified unit has a Great Leader
	# Returns False if the specified unit does not have a Great Leader
	bHasGreatLeader = False
	
	# Loop through promotions
	for iProm in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if CheckUnit.isHasPromotion(iProm):
			# Does the unit have a leader?
			if iProm == gc.getInfoTypeForString("PROMOTION_LEADER"):
				bHasGreatLeader = True
				break
	
	return bHasGreatLeader


def standardPopup(pTitle, pText, iPlayer):
	# Orion's Standard Functions
	# Displays a standard popup message for the specified Player.
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	spTitle = u"<font=4b>" + localText.getText(pTitle, ()) + u"</font>"
	spBody = u"<font=3>" + localText.getText(pText, ()) + u"</font>"
	popupInfo.setText(spTitle + u"\n\n" + spBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)
	
	
def isCapitalCity(iCityOwner, ixCity):
	# Orion's Standard Functions
	# Returns True if the specified City is the capital city for the specified player. 
	# Returns False if the specified City is not the capital city for the specified player. 
	CityIsCapital = False
	pPlayer = gc.getPlayer(iCityOwner)
	
	if pPlayer.isAlive() and not pPlayer.isBarbarian():
		iRefCityList = PyPlayer(iCityOwner).getCityList()
		MyCapitalCity = pPlayer.getCapitalCity()
		
		for pyCity in iRefCityList:
			pCity = pyCity.GetCy()
			iCity = pCity.getID()
			if pCity.isCapital():
				if iCity == ixCity:
					#CyInterface().addImmediateMessage("A", "")
					CityIsCapital = True
					break
	
	return CityIsCapital
	

def playersHaveMet(iPlayer1, iPlayer2):	
	# Orion's Standard Functions
	# Returns True if the specified players have met.
	# Returns False if the specified players have not met.
	#Team1 = gc.getTeam(gc.getPlayer(iPlayer1).getTeam())
	#Team2 = gc.getTeam(gc.getPlayer(iPlayer2).getTeam())
	Player1 = gc.getPlayer(iPlayer1)
	Player2 = gc.getPlayer(iPlayer2)
	iTeam1 = Player1.getTeam()
	iTeam2 = Player2.getTeam()
	Team1 = gc.getTeam(iTeam1)
	Team2 = gc.getTeam(iTeam2)
	szHaveMet = False
	
	# Players must have met each other
	if Team1.isHasMet(iTeam2):
		szHaveMet = True
	
	return szHaveMet
	

def playersAtWar(iPlayer1, iPlayer2):
	# Orion's Standard Functions
	# Returns True if iPlayer1 and iPlayer2 are at War
	# Returns False if iPlayer1 and iPlayer2 are not at War
	Player1 = gc.getPlayer(iPlayer1)
	Player2 = gc.getPlayer(iPlayer2)
	iTeam1 = Player1.getTeam()
	iTeam2 = Player2.getTeam()
	Team1 = gc.getTeam(iTeam1)
	Team2 = gc.getTeam(iTeam2)
	
	# Can't be the same player
	if iPlayer1 != iPlayer2:
		# Can't be on the same team
		if iTeam1 != iTeam2:
			# Players must have met each other
			if Team1.isHasMet(iTeam2):
				# Already be at war?
				if Team1.isAtWar(iTeam2):
					return True
	
	return False


def StateOfWarExists(iPlayer, iOtherPlayer):
	# Orion's Mine Warefare Mod
	AtWarWithOtherPlayer = False
	
	for iXPlayer in range(gc.getMAX_PLAYERS()):
		XPlayer = gc.getPlayer(iXPlayer)
		
		if not XPlayer.isNone() and not XPlayer.isBarbarian() and XPlayer.isAlive():
			if iXPlayer != iPlayer:
				if iOtherPlayer == iXPlayer:
					XPlayerTeam = gc.getTeam(XPlayer.getTeam())
					
					if XPlayerTeam.isAtWar(iPlayer):
						AtWarWithOtherPlayer = True
						break
	
	return AtWarWithOtherPlayer
	
	
def canPlayersDeclareWar(iPlayer1, iPlayer2):
	# Orion's Standard Functions
	# Returns True if the specified players can declare war on each other.
	# Returns False if the specified players can't declare war on each other.
	Player1 = gc.getPlayer(iPlayer1)
	Player2 = gc.getPlayer(iPlayer2)
	iTeam1 = Player1.getTeam()
	iTeam2 = Player2.getTeam()
	Team1 = gc.getTeam(iTeam1)
	Team2 = gc.getTeam(iTeam2)
	szCanDeclareWar = False
	
	# Can't be the same player
	if iPlayer1 != iPlayer2:
		# Can't be on the same team
		if iTeam1 != iTeam2:
			# Players must have met each other
			if Team1.isHasMet(iTeam2):
				# Can't already be at war
				if not Team1.isAtWar(iTeam2):
					#CyInterface().addImmediateMessage("Can Declare War!", "")
					szCanDeclareWar = True
	
	return szCanDeclareWar
	
	
def startWar(iPlayer1, iPlayer2):
	# Orion's Standard Functions
	# Causes iPlayer1 to declare war on iPlayer2 
	Player1 = gc.getPlayer(iPlayer1)
	Player2 = gc.getPlayer(iPlayer2)
	iTeam1 = Player1.getTeam()
	iTeam2 = Player2.getTeam()
	Team1 = gc.getTeam(iTeam1)
	Team2 = gc.getTeam(iTeam2)
	
	# Players must have met each other
	if Team1.isHasMet(iTeam2):
		# Can't already be at war
		if not Team1.isAtWar(iTeam2):
			# Can't be on the same team
			if iTeam1 != iTeam2:
				# Declare War
				Team1.declareWar(iTeam2, False, 1)
	
	
def isHasPermanentAlliance(iPlayer):
	# Orion's Standard Functions
	# Returns True if the specified player has a permanent alliance.
	zHasAlliance = False
	#pPlayer = gc.getPlayer(iPlayer)
	#iTeam = pPlayer.getTeam()
	#Team = gc.getTeam(iTeam)
	Team = gc.getTeam(gc.getPlayer(iPlayer).getTeam())
	
	if Team.getNumMembers() > 1:
		zHasAlliance = True
	
	return zHasAlliance
	
	
def isHasSharedPermanentAlliance(iPlayer1, iPlayer2):
	# Orion's Standard Functions
	# Returns True if both players share a permanent alliance.
	zHasSharedAlliance = False
	iTeam1 = gc.getPlayer(iPlayer1).getTeam()
	iTeam2 = gc.getPlayer(iPlayer2).getTeam()
	
	if iTeam1 == iTeam2:
		zHasSharedAlliance = True
	
	return zHasSharedAlliance


def hasTechCheck(iPlayer, iTechType):
	# Orion's Standard Functions
	# Returns True if the player has the specified tech type.
	# Returns False if the player does not have the specified tech type.
	pPlayer = gc.getPlayer(iPlayer)
	TechCheck = False
	
	for iTech in range(gc.getNumTechInfos()):
		# Does the player have this tech?
		if gc.getTeam(pPlayer.getTeam()).isHasTech(iTech):
			if iTech == iTechType:
				TechCheck = True
				break
	
	return TechCheck
	
	
def hasUnitType(iPlayer, iUnitType):
	# Orion's Standard Functions
	# Returns True if the player has the specified unit type
	# Returns False if the player does not have the specified unit type
	xRefUnitList = PyPlayer(iPlayer).getUnitList()
	
	for pyUnit in xRefUnitList:
		pyUnitType = pyUnit.getUnitType()
		
		if (pyUnitType == iUnitType):
			return True
	
	return False


def getUnitEra(pUnitXXType):
	# Orion's Standard Functions
	# Returns the Era for a specified Unit type
	# Identify a valid Unit
	if pUnitXXType != -1:
		# Get the Unit Info
		pUnitInfo = gc.getUnitInfo(pUnitXXType)
		
		if pUnitInfo != -1:
			# Get the prerequisit Tech for the Unit
			iTechType = pUnitInfo.getPrereqAndTech()
			
			if iTechType != -1:
				# Get the Tech Era
				XiEra = gc.getTechInfo(iTechType).getEra()
				
				if XiEra != -1:
					return XiEra
			
			else:
				XiEra = 0
				return XiEra
	
def hasNotExceedMaxUnitCount(iPlayer, iUnitType):
	pPlayer = gc.getPlayer(iPlayer)
	iUnitClassType = gc.getUnitInfo(iUnitType).getUnitClassType()
	pUnit = getUnit(iUnitType)
	
	if iUnitClassType != None:
		iTotalUnitClassCount = pPlayer.getUnitClassCount(iUnitClassType)
		
		if iTotalUnitClassCount > 0:
			iMaxPlayerUnitInstance = gc.getUnitClassInfo(pUnit.getUnitClassType()).getMaxPlayerInstances() 
			
			if iMaxPlayerUnitInstance != -1:
				if iTotalUnitClassCount <= iMaxPlayerUnitInstance:
					return True
	
	return False
	
	
def getTotalUnitCount(iPlayer, iUnitType):
	# Orion's Standard Functions
	# Returns the total number of specified units fielded by the specified player. 
	pPlayer = gc.getPlayer(iPlayer)
	iUnitClassType = gc.getUnitInfo(iUnitType).getUnitClassType()
	if iUnitClassType != None:
		iTotalUnitClassCount = pPlayer.getUnitClassCount(iUnitClassType)
	
	else:
		iTotalUnitClassCount = 0
	
	return iTotalUnitClassCount
	

def hasOpenBordersAgreement(iPlayerA, iPlayerB):
	# Orion's Inquisition Mod
	#Check for Open Borders Agreement between two players.
	#Returns True if Open Borders Agreement Exists.
	#Returns False if Open Borders Agreement does not Exist.
	MyAgreement = False
	
	for iMyDeal in range(gc.getGame().getIndexAfterLastDeal()):
		deal = gc.getGame().getDeal(iMyDeal)
		if ((deal.getFirstPlayer() == iPlayerA and deal.getSecondPlayer() == iPlayerB) and not deal.isNone() or (deal.getSecondPlayer() == iPlayerB and deal.getFirstPlayer() == iPlayerA)) and not deal.isNone():
			for jTrade in range(deal.getLengthFirstTrades()):
				tradeData = deal.getFirstTrade(jTrade)
				if tradeData.ItemType == TradeableItems.TRADE_OPEN_BORDERS:
					MyAgreement = True
					break
	
	return MyAgreement
	
	
def closeOpenBordersAgreement(iPlayerA, iPlayerB):
	# Orion's Mine Warefare Mod
	# Check for Open Borders Agreement between two players.
	# Closes Open Borders Agreement if one exists.
	
	for iMyDeal in range(gc.getGame().getIndexAfterLastDeal()):
		deal = gc.getGame().getDeal(iMyDeal)
		if ((deal.getFirstPlayer() == iPlayerA and deal.getSecondPlayer() == iPlayerB) and not deal.isNone() or (deal.getSecondPlayer() == iPlayerB and deal.getFirstPlayer() == iPlayerA)) and not deal.isNone():
			for jTrade in range(deal.getLengthFirstTrades()):
				tradeData = deal.getFirstTrade(jTrade)
				if tradeData.ItemType == TradeableItems.TRADE_OPEN_BORDERS:
					deal.kill()
					break


def doRemoveBuilding(iPlayer, intBuilding):
	# Orion's Cyber Warfare Mod
	# Removes one Building from a city that has the specified Building
	
	pPlayer = gc.getPlayer(iPlayer)
	iRefCityList = PyPlayer(iPlayer).getCityList()
	
	if iStateReligion != -1:
		for pyCity in iRefCityList:
			pCity = pyCity.GetCy()
			#iCity = pCity.getID()
			
			if pCity.isHasBuilding(intBuilding):
				pCity.setNumRealBuilding(intBuilding, 0)
				break
	
	return
	
#def kickNavalUnitOutOfCity(pUnit):
#	# Orion's Icebreaker Mod
#	# Force Naval Unit out of City to an adjacent water plot
#	# Note:  Unit must be on a water plot for the calculatePathDistance command to work
#	# Assumes the unit owner is an AI Player 
#	iOwner = pUnit.getOwner()
#	AIpPlayer = gc.getPlayer(iOwner)
#	FoundAdjacentPlot = False
#	iX = pUnit.getX()
#	iY = pUnit.getY()
#	pUnitPlot = CyMap().plot(iX, iY)
#	# Leave unit with one less movement point.
#	pUnit.setMoves(pUnit.maxMoves()-1)
#	# Limit iDirection to 8
#	iDirection = CyGame().getSorenRandNum(8,"")
#	
#	# Loop through adjacent plots
#	for iDirection in range(DirectionTypes.NUM_DIRECTION_TYPES):
#		pAdjacentPlot = plotDirection( iX, iY, DirectionTypes(iDirection))
#		pAdjacentPlotiOwner = pAdjacentPlot.getOwner()
#		
#		# make sure plot is not None
#		if not pAdjacentPlot.isNone():
#			# Exclude the city plot the Mine is currently on.
#			if pAdjacentPlot != pUnitPlot:
#				# LoopPlot must be unowned or owned by the AI Player
#				if pAdjacentPlotiOwner == -1 or pAdjacentPlotiOwner == iOwner:
#					# Make sure plot is not impassable (ICE)
#					if not pAdjacentPlot.isImpassable():
#						# Make sure plot is not ICE
#						if not pAdjacentPlot.getFeatureType() == gc.getInfoTypeForString("FEATURE_ICE"):
#							# Make sure this is a water plot
#							if pAdjacentPlot.isWater():
#								# Make sure the water plot is next to land
#								if pAdjacentPlot.isAdjacentToLand():
#									# Make Sure the plot is not a fresh water lake
#									if not pAdjacentPlot.isLake():
#										#CyInterface().addImmediateMessage("A", "")
#										FoundAdjacentPlot = True
#										MyAdjacentPlot = pAdjacentPlot
#										break
#	
#	if FoundAdjacentPlot:
#		#CyInterface().addImmediateMessage("Found Plot!", "")
#		# Move the naval unit out of the city to the adjacent  plot
#		pUnit.setXY(MyAdjacentPlot.getX(), MyAdjacentPlot.getY(), False, True, False)
#		#pUnit.getGroup().pushMission(MissionTypes.MISSION_MOVE_TO, MyAdjacentPlot.getX(), MyAdjacentPlot.getY(), 0, False, True, MissionAITypes.NO_MISSIONAI, pUnitPlot, pUnit)
#		#CyMap().resetPathDistance()


def isProjectComplete(iProject):
	# Orion's Standard Functions
	ProjectComplete = False
	
	for iPlayer in range(gc.getMAX_PLAYERS()):
		pPlayer = gc.getPlayer(iPlayer)
		iTeam = gc.getTeam(pPlayer.getTeam())
		
		if iTeam.getProjectCount(iProject) > 0:
			ProjectComplete = True
			break
	
	return ProjectComplete


# get list of leader traits as comma separated string
def getDefaultTraitList(iPlayer):
	pPlayer = gc.getPlayer(iPlayer)
	sTraits = ""
	bFirst = True
	for i in range(gc.getNumTraitInfos()):
		if gc.getLeaderHeadInfo(pPlayer.getLeaderType()).hasTrait(i):
			if bFirst:
				bFirst = False
			else:
				sTraits += ", "
			
			sTraits += gc.getTraitInfo(i).getType()[6:].capitalize()
	
	return sTraits
	
	
def getMysteryCount(iPlayer):
	pPlayer = gc.getPlayer(iPlayer)
	iRefCityList = PyPlayer(iPlayer).getCityList()
	pCivilization = gc.getCivilizationInfo(pPlayer.getCivilizationType())
	iSpecialBldMysteries = gc.getInfoTypeForString("SPECIALBUILDING_MYSTERIES")
	intMysteryCount = 0
	
	for pyCity in iRefCityList:
		pCity = pyCity.GetCy()
		#iCity = pCity.getID()
		
		for iBuildingClass in range(gc.getNumBuildingClassInfos()):
			kBuilding = gc.getBuildingInfo(pCivilization.getCivilizationBuildings(iBuildingClass))
			
			if kBuilding != None:
				iBuilding = pCivilization.getCivilizationBuildings(iBuildingClass)
				
				if gc.getBuildingInfo(iBuilding).getSpecialBuildingType() == iSpecialBldMysteries:
					if pCity.isHasBuilding(iBuilding):
						intMysteryCount += 1
	
	return intMysteryCount
	
	
# End Orion's Standard Functions
